package com.adamrocker.android.bitmapndk;

import java.io.IOException;
import java.io.InputStream;
import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ImageView;
import android.widget.TextView;

public class BitmapNdkActivity extends Activity {
    static {
        System.loadLibrary("bitmap-ndk");
    }
    private ImageView mImage;
    private Bitmap mOriginalBitmap;
    private TextView mJavaResult;
    private TextView mNativeResult;
    
    private OnClickListener mOnClickListener = new OnClickListener() {
        public void onClick(View v) {
            Bitmap bit = null;
            switch (v.getId()) {
                case R.id.java_btn:
                    bit = mOriginalBitmap.copy(Bitmap.Config.ARGB_8888, true);
                    effectJava(bit);//Javaで画像処理
                    break;
                case R.id.native_btn:
                    bit = mOriginalBitmap.copy(Bitmap.Config.ARGB_8888, true);
                    callNative(bit);//Nativeで画像処理
                    break;
                case R.id.original_btn:
                    bit = mOriginalBitmap;
                    clearResults();
                    break;
            }
            mImage.setImageBitmap(bit);
        }
    };

    private native void effectNative(Bitmap bit);
    
    private void callNative(Bitmap bit) {
        synchronized (mImage) {
            long start = System.currentTimeMillis();
            effectNative(bit);
            long end = System.currentTimeMillis();
            setNativeResult(end - start);
        }
    }
    
    private void effectJava(Bitmap bit) {
        synchronized (mImage) {
            long start = System.currentTimeMillis();
            int width = bit.getWidth();
            int height = bit.getHeight();
            int[] pixels = new int[width * height];//画素
            bit.getPixels(pixels, 0, width, 0, 0, width, height);//画素を取得
            effect(pixels, width, height, 3);//画像処理
            bit.setPixels(pixels, 0, width, 0, 0, width, height);//画素を設定
            long end = System.currentTimeMillis();
            setJavaResult(end - start);//処理時間を表示
        }
    }
    
    //グレースケールにして、ネガ・ポジ反転
    private void effect(int[] pixels, int width, int height, int block) {
        for (int y = 0; y < height; y++) {
            int line = y * width;
            for (int x = 0; x < width; x++) {
                int index = line + x;
                int color = pixels[index];
                int r = (color & 0x00ff0000) >> 16;
                int g = (color & 0x0000ff00) >> 8;
                int b = (color & 0x000000ff);
                //NTSC加重平均によるグレースケール値
                double gray = 0.298912 * r + 0.586611 * g + 0.114478 * b;
                
                //ネガ・ポジ反転
                r = (int) (255.0 - gray);
                g = r;
                b = g;

                pixels[index] = 0xff000000 | (r << 16) | (g << 8) | b;
            }
        }
    }
    
    private void clearResults() {
        setJavaResult(0);
        setNativeResult(0);
    }
    
    private void setJavaResult(long msec) {
        mJavaResult.setText(msec + " ms");
    }
    
    private void setNativeResult(long msec) {
        mNativeResult.setText(msec + " ms");
    }
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);//レイアウトの設定
        View btn = findViewById(R.id.java_btn);
        btn.setOnClickListener(mOnClickListener);//ボタン処理
        btn = findViewById(R.id.native_btn);
        btn.setOnClickListener(mOnClickListener);//ボタン処理
        btn = findViewById(R.id.original_btn);
        btn.setOnClickListener(mOnClickListener);//ボタン処理
        mJavaResult = (TextView)findViewById(R.id.java_tv);//Java実行時間
        mNativeResult = (TextView)findViewById(R.id.native_tv);//Native実行時間
        mImage = (ImageView)findViewById(R.id.preview_iv);//画像表示
        InputStream is;
        try {
            //画像読込み
            is = getAssets().open("sample.jpg");
            mOriginalBitmap = BitmapFactory.decodeStream(is);
            mImage.setImageBitmap(mOriginalBitmap);
            is.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
   
}